@@ -1,4 +1,5 @@
 class BaseError(Exception):
+
     def __init__(self, messages):
         if not isinstance(messages, (list, tuple, dict)):
             messages = [messages]
@@ -11,21 +12,22 @@ class BaseError(Exception):
     def clean_messages(self, messages):
         if isinstance(messages, dict):
             clean_messages = {}
-            for k, v in messages.iteritems():
-                if isinstance(v, ValidationError):
-                    v = v.messages
-                clean_messages[k] = v
+            for key, value in messages.iteritems():
+                if isinstance(value, ValidationError):
+                    value = value.messages
+                clean_messages[key] = value
         else:
             clean_messages = []
-            for v in messages:
-                if isinstance(v, ValidationError):
-                    v = v.messages
-                clean_messages.append(v)
+            for message in messages:
+                if isinstance(message, ValidationError):
+                    message = message.messages
+                clean_messages.append(message)
 
         return clean_messages
 
 
 class ConversionError(BaseError, TypeError):
+
     """ Exception raised when data cannot be converted to the correct python type """
     pass
 
@@ -35,6 +37,7 @@ class ModelConversionError(ConversionError):
 
 
 class ValidationError(BaseError, ValueError):
+
     """Exception raised when invalid data is encountered."""
     pass
 
@@ -44,5 +47,11 @@ class ModelValidationError(ValidationError):
 
 
 class StopValidation(ValidationError):
+
     """Exception raised when no more validation need occur."""
     pass
+
+
+class MockCreationError(ValueError):
+    """Exception raised when a mock value cannot be generated."""
+    pass
