@@ -4,8 +4,21 @@ import datetime
 import decimal
 import itertools
 import functools
+import random
+import string
 
-from ..exceptions import StopValidation, ValidationError, ConversionError
+from ..exceptions import (
+    StopValidation, ValidationError, ConversionError, MockCreationError
+)
+
+
+def fill_template(template, min_length, max_length):
+    return template % random_string(
+        get_value_in(
+            min_length,
+            max_length,
+            padding=len(template) - 2,
+            required_length=1))
 
 
 def force_unicode(obj, encoding='utf-8'):
@@ -18,17 +31,49 @@ def force_unicode(obj, encoding='utf-8'):
     return obj
 
 
+def get_range_endpoints(min_length, max_length, padding=0, required_length=0):
+    if min_length is None and max_length is None:
+        min_length = 0
+        max_length = 16
+    elif min_length is None:
+        min_length = 0
+    elif max_length is None:
+        max_length = max(min_length * 2, 16)
+
+    if padding:
+        max_length = max_length - padding
+        min_length = max(min_length - padding, 0)
+
+    if max_length < required_length:
+        raise MockCreationError(
+            'This field is too short to hold the mock data')
+
+    min_length = max(min_length, required_length)
+
+    return min_length, max_length
+
+
+def get_value_in(min_length, max_length, padding=0, required_length=0):
+    return random.randint(
+        *get_range_endpoints(min_length, max_length, padding, required_length))
+
+
+def random_string(length, chars=string.letters + string.digits):
+    return ''.join(random.choice(chars) for _ in range(length))
+
+
 _last_position_hint = -1
 _next_position_hint = itertools.count()
 
 
 class TypeMeta(type):
+
     """
     Meta class for BaseType. Merges `MESSAGES` dict and accumulates
     validator methods.
     """
 
-    def __new__(cls, name, bases, attrs):
+    def __new__(mcs, name, bases, attrs):
         messages = {}
         validators = []
 
@@ -50,10 +95,11 @@ class TypeMeta(type):
 
         attrs["_validators"] = validators
 
-        return type.__new__(cls, name, bases, attrs)
+        return type.__new__(mcs, name, bases, attrs)
 
 
 class BaseType(TypeMeta('BaseTypeBase', (object, ), {})):
+
     """A base class for Types in a Schematics model. Instances of this
     class may be added to subclasses of ``Model`` to define a model schema.
 
@@ -101,6 +147,7 @@ class BaseType(TypeMeta('BaseTypeBase', (object, ), {})):
     def __init__(self, required=False, default=None, serialized_name=None,
                  choices=None, validators=None, deserialize_from=None,
                  serialize_when_none=None, messages=None):
+        super(BaseType, self).__init__()
         self.required = required
         self._default = default
         self.serialized_name = serialized_name
@@ -118,6 +165,9 @@ class BaseType(TypeMeta('BaseTypeBase', (object, ), {})):
     def __call__(self, value):
         return self.to_native(value)
 
+    def _mock(self, context=None):
+        return None
+
     @property
     def default(self):
         default = self._default
@@ -125,12 +175,12 @@ class BaseType(TypeMeta('BaseTypeBase', (object, ), {})):
             default = self._default()
         return default
 
-    def to_primitive(self, value):
+    def to_primitive(self, value, context=None):
         """Convert internal data to a value safe to serialize.
         """
         return value
 
-    def to_native(self, value):
+    def to_native(self, value, context=None):
         """
         Convert untrusted data to a richer Python construct.
         """
@@ -156,10 +206,10 @@ class BaseType(TypeMeta('BaseTypeBase', (object, ), {})):
         for validator in self.validators:
             try:
                 validator(value)
-            except ValidationError as e:
-                errors.extend(e.messages)
+            except ValidationError as exc:
+                errors.extend(exc.messages)
 
-                if isinstance(e, StopValidation):
+                if isinstance(exc, StopValidation):
                     break
 
         if errors:
@@ -173,30 +223,45 @@ class BaseType(TypeMeta('BaseTypeBase', (object, ), {})):
         if self.choices is not None:
             if value not in self.choices:
                 raise ValidationError(self.messages['choices']
-                    .format(unicode(self.choices)))
+                                      .format(unicode(self.choices)))
+
+    def mock(self, context=None):
+        if not self.required and not random.choice([True, False]):
+            return self.default
+        if self.choices is not None:
+            return random.choice(self.choices)
+        return self._mock(context)
 
 
 class UUIDType(BaseType):
+
     """A field that stores a valid UUID value.
     """
+    MESSAGES = {
+        'convert': u"Couldn't interpret value as UUID.",
+    }
 
-    def to_native(self, value):
+    def _mock(self, context=None):
+        return uuid.uuid4()
+
+    def to_native(self, value, context=None):
         if not isinstance(value, uuid.UUID):
-            value = uuid.UUID(value)
+            try:
+                value = uuid.UUID(value)
+            except (AttributeError, TypeError, ValueError):
+                raise ConversionError(self.messages['convert'])
         return value
 
-    def to_primitive(self, value):
+    def to_primitive(self, value, context=None):
         return str(value)
 
 
 class IPv4Type(BaseType):
+
     """ A field that stores a valid IPv4 address """
 
-    def __init__(self, auto_fill=False, **kwargs):
-        super(IPv4Type, self).__init__(**kwargs)
-
-    def _jsonschema_type(self):
-        return 'string'
+    def _mock(self, context=None):
+        return '.'.join(str(random.randrange(256)) for _ in range(4))
 
     @classmethod
     def valid_ip(cls, addr):
@@ -205,7 +270,7 @@ class IPv4Type(BaseType):
         except AttributeError:
             return False
         try:
-            return len(addr) == 4 and all(int(octet) < 256 for octet in addr)
+            return len(addr) == 4 and all(0 <= int(octet) < 256 for octet in addr)
         except ValueError:
             return False
 
@@ -219,19 +284,9 @@ class IPv4Type(BaseType):
             raise ValidationError(error_msg)
         return True
 
-    def _jsonschema_format(self):
-        return 'ip-address'
-
-    @classmethod
-    def _from_jsonschema_formats(self):
-        return ['ip-address']
-
-    @classmethod
-    def _from_jsonschema_types(self):
-        return ['string']
-
 
 class StringType(BaseType):
+
     """A unicode string field. Default minimum length is one. If you want to
     accept empty strings, init with ``min_length`` 0.
     """
@@ -252,7 +307,10 @@ class StringType(BaseType):
 
         super(StringType, self).__init__(**kwargs)
 
-    def to_native(self, value):
+    def _mock(self, context=None):
+        return random_string(get_value_in(self.min_length, self.max_length))
+
+    def to_native(self, value, context=None):
         if value is None:
             return None
 
@@ -281,6 +339,7 @@ class StringType(BaseType):
 
 
 class URLType(StringType):
+
     """A field that validates input as an URL.
 
     If verify_exists=True is passed the validate function will make sure
@@ -305,6 +364,10 @@ class URLType(StringType):
         self.verify_exists = verify_exists
         super(URLType, self).__init__(**kwargs)
 
+    def _mock(self, context=None):
+        return fill_template('http://a%s.ZZ', self.min_length,
+                             self.max_length)
+
     def validate_url(self, value):
         if not URLType.URL_REGEX.match(value):
             raise StopValidation(self.messages['invalid_url'])
@@ -318,6 +381,7 @@ class URLType(StringType):
 
 
 class EmailType(StringType):
+
     """A field that validates input as an E-Mail-Address.
     """
 
@@ -336,12 +400,17 @@ class EmailType(StringType):
         re.IGNORECASE
     )
 
+    def _mock(self, context=None):
+        return fill_template('%s@example.com', self.min_length,
+                             self.max_length)
+
     def validate_email(self, value):
         if not EmailType.EMAIL_REGEX.match(value):
             raise StopValidation(self.messages['email'])
 
 
 class NumberType(BaseType):
+
     """A number field.
     """
 
@@ -360,28 +429,32 @@ class NumberType(BaseType):
 
         super(NumberType, self).__init__(**kwargs)
 
-    def to_native(self, value):
+    def _mock(self, context=None):
+        return get_value_in(self.min_value, self.max_value)
+
+    def to_native(self, value, context=None):
         try:
             value = self.number_class(value)
         except (TypeError, ValueError):
             raise ConversionError(self.messages['number_coerce']
-                .format(self.number_type.lower()))
+                                  .format(self.number_type.lower()))
 
         return value
 
     def validate_range(self, value):
         if self.min_value is not None and value < self.min_value:
             raise ValidationError(self.messages['number_min']
-                .format(self.number_type, self.min_value))
+                                  .format(self.number_type, self.min_value))
 
         if self.max_value is not None and value > self.max_value:
             raise ValidationError(self.messages['number_max']
-                .format(self.number_type, self.max_value))
+                                  .format(self.number_type, self.max_value))
 
         return value
 
 
 class IntType(NumberType):
+
     """A field that validates input as an Integer
     """
 
@@ -392,8 +465,10 @@ class IntType(NumberType):
 
 
 class LongType(NumberType):
+
     """A field that validates input as a Long
     """
+
     def __init__(self, *args, **kwargs):
         super(LongType, self).__init__(number_class=long,
                                        number_type='Long',
@@ -401,8 +476,10 @@ class LongType(NumberType):
 
 
 class FloatType(NumberType):
+
     """A field that validates input as a Float
     """
+
     def __init__(self, *args, **kwargs):
         super(FloatType, self).__init__(number_class=float,
                                         number_type='Float',
@@ -410,13 +487,14 @@ class FloatType(NumberType):
 
 
 class DecimalType(BaseType):
+
     """A fixed-point decimal number field.
     """
 
     MESSAGES = {
         'number_coerce': 'Number failed to convert to a decimal',
-        'number_min': u"Value should be greater than {}",
-        'number_max': u"Value should be less than {}",
+        'number_min': u"Value should be greater than {0}",
+        'number_max': u"Value should be less than {0}",
     }
 
     def __init__(self, min_value=None, max_value=None, **kwargs):
@@ -424,10 +502,13 @@ class DecimalType(BaseType):
 
         super(DecimalType, self).__init__(**kwargs)
 
-    def to_primitive(self, value):
+    def _mock(self, context=None):
+        return get_value_in(self.min_value, self.max_value)
+
+    def to_primitive(self, value, context=None):
         return unicode(value)
 
-    def to_native(self, value):
+    def to_native(self, value, context=None):
         if not isinstance(value, decimal.Decimal):
             if not isinstance(value, basestring):
                 value = unicode(value)
@@ -458,7 +539,10 @@ class HashType(BaseType):
         'hash_hex': u"Hash value is not hexadecimal.",
     }
 
-    def to_native(self, value):
+    def _mock(self, context=None):
+        return random_string(self.LENGTH, string.hexdigits)
+
+    def to_native(self, value, context=None):
         if len(value) != self.LENGTH:
             raise ValidationError(self.messages['hash_length'])
         try:
@@ -469,6 +553,7 @@ class HashType(BaseType):
 
 
 class MD5Type(HashType):
+
     """A field that validates input as resembling an MD5 hash.
     """
 
@@ -476,6 +561,7 @@ class MD5Type(HashType):
 
 
 class SHA1Type(HashType):
+
     """A field that validates input as resembling an SHA1 hash.
     """
 
@@ -483,6 +569,7 @@ class SHA1Type(HashType):
 
 
 class BooleanType(BaseType):
+
     """A boolean field type. In addition to ``True`` and ``False``, coerces these
     values:
 
@@ -494,13 +581,19 @@ class BooleanType(BaseType):
     TRUE_VALUES = ('True', 'true', '1')
     FALSE_VALUES = ('False', 'false', '0')
 
-    def to_native(self, value):
+    def _mock(self, context=None):
+        return random.choice([True, False])
+
+    def to_native(self, value, context=None):
         if isinstance(value, basestring):
             if value in self.TRUE_VALUES:
                 value = True
             elif value in self.FALSE_VALUES:
                 value = False
 
+        if isinstance(value, int) and value in [0, 1]:
+            value = bool(value)
+
         if not isinstance(value, bool):
             raise ConversionError(u'Must be either true or false.')
 
@@ -508,6 +601,7 @@ class BooleanType(BaseType):
 
 
 class DateType(BaseType):
+
     """Defaults to converting to and from ISO8601 date values.
     """
 
@@ -520,7 +614,14 @@ class DateType(BaseType):
         self.serialized_format = self.SERIALIZED_FORMAT
         super(DateType, self).__init__(**kwargs)
 
-    def to_native(self, value):
+    def _mock(self, context=None):
+        return datetime.datetime(
+            year=random.randrange(600) + 1900,
+            month=random.randrange(12) + 1,
+            day=random.randrange(28) + 1,
+        )
+
+    def to_native(self, value, context=None):
         if isinstance(value, datetime.date):
             return value
 
@@ -529,11 +630,12 @@ class DateType(BaseType):
         except (ValueError, TypeError):
             raise ConversionError(self.messages['parse'].format(value))
 
-    def to_primitive(self, value):
+    def to_primitive(self, value, context=None):
         return value.strftime(self.serialized_format)
 
 
 class DateTimeType(BaseType):
+
     """Defaults to converting to and from ISO8601 datetime values.
 
     :param formats:
@@ -555,7 +657,7 @@ class DateTimeType(BaseType):
         """
 
         """
-        if isinstance(format, basestring):
+        if isinstance(formats, basestring):
             formats = [formats]
         if formats is None:
             formats = self.DEFAULT_FORMATS
@@ -565,35 +667,50 @@ class DateTimeType(BaseType):
         self.serialized_format = serialized_format
         super(DateTimeType, self).__init__(**kwargs)
 
-    def to_native(self, value):
+    def _mock(self, context=None):
+        return datetime.datetime(
+            year=random.randrange(600) + 1900,
+            month=random.randrange(12) + 1,
+            day=random.randrange(28) + 1,
+            hour=random.randrange(24),
+            minute=random.randrange(60),
+            second=random.randrange(60),
+            microsecond=random.randrange(1000000),
+        )
+
+    def to_native(self, value, context=None):
         if isinstance(value, datetime.datetime):
             return value
 
-        for format in self.formats:
+        for fmt in self.formats:
             try:
-                return datetime.datetime.strptime(value, format)
+                return datetime.datetime.strptime(value, fmt)
             except (ValueError, TypeError):
                 continue
         raise ConversionError(self.messages['parse'].format(value))
 
-    def to_primitive(self, value):
+    def to_primitive(self, value, context=None):
         if callable(self.serialized_format):
             return self.serialized_format(value)
         return value.strftime(self.serialized_format)
 
 
 class GeoPointType(BaseType):
+
     """A list storing a latitude and longitude.
     """
 
-    def to_native(self, value):
+    def _mock(self, context=None):
+        return (random.randrange(-90, 90), random.randrange(-90, 90))
+
+    def to_native(self, value, context=None):
         """Make sure that a geo-value is of type (x, y)
         """
         if not len(value) == 2:
             raise ValueError('Value must be a two-dimensional point')
         if isinstance(value, dict):
-            for v in value.values():
-                if not isinstance(v, (float, int)):
+            for val in value.values():
+                if not isinstance(val, (float, int)):
                     raise ValueError('Both values in point must be float or int')
         elif isinstance(value, (list, tuple)):
             if (not isinstance(value[0], (float, int)) or
@@ -603,3 +720,118 @@ class GeoPointType(BaseType):
             raise ValueError('GeoPointType can only accept tuples, lists, or dicts')
 
         return value
+
+
+class MultilingualStringType(BaseType):
+
+    """
+    A multilanguage string field, stored as a dict with {'locale': 'localized_value'}.
+
+    Minimum and maximum lengths apply to each of the localized values.
+
+    At least one of ``default_locale`` or ``context['locale']`` must be defined
+    when calling ``.to_primitive``.
+
+    """
+
+    allow_casts = (int, str)
+
+    MESSAGES = {
+        'convert': u"Couldn't interpret value as string.",
+        'max_length': u"String value in locale %s is too long.",
+        'min_length': u"String value in locale %s is too short.",
+        'locale_not_found': u"No requested locale was available.",
+        'no_locale': u"No default or explicit locales were given.",
+        'regex_locale': u"Name of locale %s did not match validation regex.",
+        'regex_localized': u"String value in locale %s did not match validation regex.",
+    }
+
+    LOCALE_REGEX = r'^[a-z]{2}(:?_[A-Z]{2})?$'
+
+    def __init__(self, regex=None, max_length=None, min_length=None,
+                 default_locale=None, locale_regex=LOCALE_REGEX, **kwargs):
+        self.regex = re.compile(regex) if regex else None
+        self.max_length = max_length
+        self.min_length = min_length
+        self.default_locale = default_locale
+        self.locale_regex = re.compile(locale_regex) if locale_regex else None
+
+        super(MultilingualStringType, self).__init__(**kwargs)
+
+    def _mock(self, context=None):
+        return random_string(get_value_in(self.min_length, self.max_length))
+
+    def to_native(self, value, context=None):
+        """Make sure a MultilingualStringType value is a dict or None."""
+
+        if not (value is None or isinstance(value, dict)):
+            raise ValueError('Value must be a dict or None')
+
+        return value
+
+    def to_primitive(self, value, context=None):
+        """
+        Use a combination of ``default_locale`` and ``context['locale']`` to return
+        the best localized string.
+
+        """
+        if value is None:
+            return None
+
+        context_locale = None
+        if context is not None and 'locale' in context:
+            context_locale = context['locale']
+
+        # Build a list of all possible locales to try
+        possible_locales = []
+        for locale in (context_locale, self.default_locale):
+            if not locale:
+                continue
+
+            if isinstance(locale, basestring):
+                possible_locales.append(locale)
+            else:
+                possible_locales.extend(locale)
+
+        if not possible_locales:
+            raise ConversionError(self.messages['no_locale'])
+
+        for locale in possible_locales:
+            if locale in value:
+                localized = value[locale]
+                break
+        else:
+            raise ConversionError(self.messages['locale_not_found'])
+
+        if not isinstance(localized, unicode):
+            if isinstance(localized, self.allow_casts):
+                if not isinstance(localized, str):
+                    localized = str(localized)
+                localized = unicode(localized, 'utf-8')
+            else:
+                raise ConversionError(self.messages['convert'])
+
+        return localized
+
+    def validate_length(self, value):
+        for locale, localized in value.items():
+            len_of_value = len(localized) if localized else 0
+
+            if self.max_length is not None and len_of_value > self.max_length:
+                raise ValidationError(self.messages['max_length'] % locale)
+
+            if self.min_length is not None and len_of_value < self.min_length:
+                raise ValidationError(self.messages['min_length'] % locale)
+
+    def validate_regex(self, value):
+        if self.regex is None and self.locale_regex is None:
+            return
+
+        for locale, localized in value.items():
+            if self.regex is not None and self.regex.match(localized) is None:
+                raise ValidationError(
+                    self.messages['regex_localized'] % locale)
+
+            if self.locale_regex is not None and self.locale_regex.match(locale) is None:
+                raise ValidationError(
+                    self.messages['regex_locale'] % locale)
