@@ -5,7 +5,7 @@ from schematics.models import Model
 from schematics.types import StringType, LongType, IntType, MD5Type
 from schematics.types.compound import ModelType, DictType, ListType
 from schematics.types.serializable import serializable
-from schematics.transforms import blacklist, whitelist, wholelist
+from schematics.transforms import blacklist, whitelist, wholelist, export_loop
 
 
 def test_serializable():
@@ -23,6 +23,9 @@ def test_serializable():
     d = location_US.serialize()
     assert d == {"country_code": "US", "country_name": "United States"}
 
+    d = location_US.to_native()
+    assert d == {"country_code": u"US", "country_name": "United States"}
+
     location_IS = Location({"country_code": "IS"})
 
     assert location_IS.country_name == "Unknown"
@@ -30,6 +33,23 @@ def test_serializable():
     d = location_IS.serialize()
     assert d == {"country_code": "IS", "country_name": "Unknown"}
 
+    d = location_IS.to_native()
+    assert d == {"country_code": "IS", "country_name": "Unknown"}
+
+
+def test_serializable_to_native():
+    class Location(Model):
+        country_code = StringType()
+
+        @serializable
+        def country_name(self):
+            return "United States" if self.country_code == "US" else "Unknown"
+
+    loc = Location({'country_code': 'US'})
+
+    d = loc.to_native()
+    assert d == {'country_code': 'US', 'country_name': 'United States'}
+
 
 def test_serializable_with_serializable_name():
     class Location(Model):
@@ -50,7 +70,7 @@ def test_serializable_with_serializable_name():
 def test_serializable_with_custom_serializable_class():
     class PlayerIdType(LongType):
 
-        def to_primitive(self, value):
+        def to_primitive(self, value, context=None):
             return unicode(value)
 
     class Player(Model):
@@ -402,6 +422,36 @@ def test_complex_types_hiding_after_apply_role_leaves_it_empty():
     assert d == {'question_id': '1'}
 
 
+def test_serialize_none_fields_if_field_says_so():
+    class TestModel(Model):
+        inst_id = StringType(required=True, serialize_when_none=True)
+
+    q = TestModel({'inst_id': 1})
+
+    d = export_loop(TestModel, q, lambda field, value: None)
+    assert d == {'inst_id': None}
+
+
+def test_serialize_none_fields_if_export_loop_says_so():
+    class TestModel(Model):
+        inst_id = StringType(required=True, serialize_when_none=False)
+
+    q = TestModel({'inst_id': 1})
+
+    d = export_loop(TestModel, q, lambda field, value: None, print_none=True)
+    assert d == {'inst_id': None}
+
+
+def test_serialize_print_none_always_gets_you_something():
+    class TestModel(Model):
+        pass
+
+    q = TestModel()
+
+    d = export_loop(TestModel, q, lambda field, value: None, print_none=True)
+    assert d == {}
+
+
 def test_roles_work_with_subclassing():
     class Address(Model):
         private_key = StringType()
@@ -673,8 +723,28 @@ def test_role_set_operations():
                 'create': all_fields - ['id'],
                 'public': all_fields - ['password'],
                 'nospam': blacklist('password') + blacklist('email'),
+                'empty': whitelist(),
+                'everything': blacklist(),
             }
 
+    roles = User.Options.roles
+    assert len(roles['create']) == 3
+    assert len(roles['public']) == 3
+    assert len(roles['nospam']) == 2
+    assert len(roles['empty']) == 0
+    assert len(roles['everything']) == 0
+
+    # Sets sort different with different Python versions. We should be getting something back
+    # like: "whitelist('password', 'email', 'name')"
+    s = str(roles['create'])
+    assert s.startswith('whitelist(') and s.endswith(')')
+    assert sorted(s[10:-1].split(', ')) == ["'email'", "'name'", "'password'"]
+
+    # Similar, but now looking for: <Role whitelist('password', 'email', 'name')>
+    r = repr(roles['create'])
+    assert r.startswith('<Role whitelist(') and r.endswith(')>')
+    assert sorted(r[16:-2].split(', ')) == ["'email'", "'name'", "'password'"]
+
     data = {
         'id': 'NaN',
         'name': 'Arthur',
@@ -704,6 +774,19 @@ def test_role_set_operations():
         'name': 'Arthur',
     }
 
+    d = user.serialize(role='empty')
+
+    assert d is None
+
+    d = user.serialize(role='everything')
+
+    assert d == {
+        'email': 'adent@hitchhiker.gal',
+        'id': 42,
+        'name': 'Arthur',
+        'password': 'dolphins'
+    }
+
     def test_md5_type(self):
         class M(Model):
             md5 = MD5Type()
@@ -718,10 +801,11 @@ def test_role_set_operations():
         self.assertEqual(d, {
             'md5': myhash
         })
-        
+
         m2 = M(d)
         self.assertEqual(m2.md5, myhash)
 
+
 def test_serializable_with_list_and_default_role():
     class Player(Model):
         id = LongType()
